#Copyright 2006 DR0ID <dr0id@bluewin.ch> http://mypage.bluewin.ch/DR0ID
#
#
#
"""
#TODO: documentation!
"""

__author__ = "$Author: DR0ID $"
__version__= "$Revision: 35 $"
__date__   = "$Date: 2006-09-07 18:33:53 +0200 (Do, 07 Sep 2006) $"


import pygame
import random
import os


def reverse_images(images):
    rev = []
    for i in range(len(images)):
        rev.append(images[len(images)-i-1])
    return rev


def set_colorkeys(images, colorkey):
    for img in images:
        img.set_colorkey(colorkey)

def load_img_from_dir(path, optimize=1):
    print "loading images from directory ", path
    filelist = os.listdir(path)
    filelist.sort()
    images = []
    for file in filelist:
        img =  pygame.image.load(os.path.join(path,file))
        if optimize:
            if img.get_alpha() is not None:
                img = img.convert_alpha()
            else:
                img = img.convert()
        images.append(img)
        
    return images


def load_img_from_strip(file, imgcount, optimize=1):
    print "loading images from strip file ", file
    strip = pygame.image.load(file)
    sw, sh = strip.get_size()
    imgW = sw/imgcount
    images = []
    img = pygame.Surface(imgW, sh)
    srect = img.get_rect()
    for i in range(imgcount):
        
        img.fill((255,0,255,0))
        srect.move_ip(imgW, 0)
        img.blit(strip, i*imgW, srect)

        if optimize:
            if img.get_alpha() is not None:
                img = img.convert_alpha()
            else:
                img = img.convert()
        images.append(img.copy())
        
    return images


class TextSprite(pygame.sprite.Sprite):
    
    
    def __init__(self, font, pos, color, defaulttext):
        pygame.sprite.Sprite.__init__(self)
        self.text = defaulttext
        self.font = font
        self.pos = pos
        self.color = color
        self.image = None
        self.rect = None
        self.setText(self.text)
        self.startTime = pygame.time.get_ticks() # ms
        
    def setText(self, text):
        self.image = self.font.render(text,  2, self.color) 
        self.rect = self.image.get_rect(midtop = self.pos)
        self.startTime = pygame.time.get_ticks()
        
    def update(self, *arg):
        now = pygame.time.get_ticks()
        if(self.startTime!=-1 and now-self.startTime>3000):
            self.image = self.font.render(self.text,  2, self.color) 
            self.rect = self.image.get_rect(midtop = self.pos)
            self.startTime = -1
            



class Animation(pygame.sprite.Sprite):
    
    def __init__(self, centerpos, images, fps, movelist=[]  ):
        pygame.sprite.Sprite.__init__(self)
        self.images = images
        self.image = self.images[0]
        self.pos = centerpos
        self.rect = self.image.get_rect(center = self.pos)
        self.idx = 0
        self.fps = 30/fps
        self.playing = False
        self.counter = 0
        self.loop = False
        self.movelist = []#movelist
        self.operator = []
        self.notfinished = False #
        
    
    def play(self, centerpos=None, loop=False):
        if centerpos is not None:
            self.pos = centerpos    
        self.playing = True
        self.loop = loop
        for o in self.operator:
            o.init()
##        self.notfinished = True
        
        
    def update(self, dt):
##        print self.__class__, "update()"
        self.counter += 1
        if self.counter%self.fps ==0:
            if self.playing:
                self.counter %= self.fps
                self.idx += 1
                
                if self.idx==(len(self.images)-1) and self.loop==False:
                    self.playing=False
##                    self.notfinished = False
                    
                self.idx %= len(self.images)
                print self.idx
                self.image = self.images[self.idx]
                self.rect = self.image.get_rect(center = self.pos)
            if len(self.movelist)>0:
##                print self.__class__, "update, movingto", self.movelist[0]
                self.rect.move_ip(self.movelist.pop(0))
                
        for o in self.operator:
            if o.update(self.image)==0:
                self.operator.remove(o)
        
        return self.playing
    
    def draw(self, scr):
        scr.blit(self.image, self.rect)
        
    def move(self, pos, t):
##        print "moveto",self.rect.topleft, pos, t
        self.movelist = []
        x, y = pos
        steps = t*self.fps
        dx = (x-self.rect.x)/float(steps)
        dy = (y-self.rect.y)/float(steps)
        for i in range(steps):
            self.movelist.append((round(dx),round(dy)))
##        for i in self.movelist:
##            print i

            




class Operator(object):
    
    def __init__(self, fps):
        self.init()
        self.fps = 30/fps
        self.count = 0
        
    def init(self):
        pass
        
    def update(self, img):
        self.count += 1
        if self.count%self.fps ==0:
            self.count = 0
            return 1
        return 0
        

class fadeOutOp(Operator):
    
    def __init__(self, dalpha=1, fps=30):
        Operator.__init__(self, fps)
        self.da = dalpha
        
        
    def update(self, img):
        if Operator.update(self, img):
            alpha = img.get_alpha()
            alpha -= self.da
            if alpha < 0:
                return 0
            img.set_alpha(alpha)
        return 1
        
        
class fadeInOp(Operator):
    
    def __init__(self, da=1, fps = 30):
        Operator.__init__(self, fps)
        self.da = da
    
    def update(self, img):
        if Operator.update(self, img):
            alpha = img.get_alpha()
            alpha += self.da
            if alpha > 255:
                return 0
            img.set_alpha(alpha)
        return 1
        
        
class flimmerOp(Operator):
    
    def update(self, img):
        if Operator.update(self, img):
            alpha = random.randint(50, 230)
            img.set_alpha(alpha)